﻿using System;
using System.IO;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace AsyncWFDemo
{
    public partial class Form1 : Form
    {
        private readonly string sampleFilePath = AppDomain.CurrentDomain.BaseDirectory
                                                 + @"..\..\..\AsynchronousPrograming\SampleData\SampleCsvRecords.csv";

        public Form1()
        {
            InitializeComponent();
        }

        private void Button1_Click(object sender, EventArgs e)
        {
            using (var reader = new StreamReader(sampleFilePath))
            {
                var counter = 0;
                for (var i = 0; i < 1000000; i++)
                {
                    var content = reader.ReadToEnd();
                    counter += content.Length;
                }
                MessageBox.Show($"{counter} characters have been red.", "Work completed.");
            }
        }

        private async void Button2_Click(object sender, EventArgs e)
        {
            using (var reader = new StreamReader(sampleFilePath))
            {
                var counter = 0;
                for (var i = 0; i < 1000; i++)
                {
                    var content = await reader.ReadToEndAsync();
                    counter += content.Length;
                }
                MessageBox.Show($"{counter} characters have been red.", "Work completed.");
            }
        }

        // Example of bad usage of async
        private void Button3_Click(object sender, EventArgs e)
        {
            // Following code will cause a deadlock
            var jsonTask = GetFirstLineAsync();
            textBox1.Text = jsonTask.Result;
        }

        private async Task<string> GetFirstLineAsync()
        {
            using (var reader = new StreamReader(sampleFilePath))
            {
                return await reader.ReadLineAsync(); 
            }
        }

        /*
        // I. (horsi varianta) opravy -> pridani ConfigureAwait(false); 
         
        private async Task<string> GetFirstLineAsync()
        {
            using (var reader = new StreamReader(sampleFilePath))
            { 
                return await reader.ReadLineAsync().ConfigureAwait(false); 
            }
        }
        */

        /*
        // II. (lepsi varianta) opravy -> pridani async... ...await do obsluhy udalosti;      
        private async void Button3_Click(object sender, EventArgs e)
        {
            // Following code will cause a deadlock
            textBox1.Text = await GetFirstLineAsync();
        }
        */
    }
}
