﻿using System;
using System.Threading.Tasks;

namespace AsynchronousPrograming
{
    /// <summary>
    /// Basic example of asynchronous invocation 
    /// with and without return value
    /// </summary>
    public static class Example01
    {
        public static void Run()
        {
            // Simple example with no return value
            ProcedureWithNoResultAsync();
            // Do something within the Run method (no return value is awaited)
            Console.WriteLine("Performing some job in Run() method...");
            Console.ReadLine();

            // Simple example with return value
            ProcedureWithResultExampleAsync();
            Console.WriteLine("Again, performing some job in Run() method...");
            Console.ReadLine();
        }

        /// <summary>
        /// Perform some operation which can be awaited (I/O, API call, ...)
        /// </summary>
        private static async Task ProcedureWithNoResultAsync()
        {            
            Console.WriteLine("ProcedureWithNoResultAsync has started...");
            /* Task.Delay(1000) is awaited which means the caller method 
             can execute code, untill the operation is finished */
            await Task.Delay(1000);
            Console.WriteLine("ProcedureWithNoResultAsync has finished." + Environment.NewLine + 
                "Press any key to continue with the demonstration...");
        }

        /// <summary>
        /// Perform some awaitable operation which returns a value
        /// </summary>
        private static async Task ProcedureWithResultExampleAsync()
        {
            var result = await ProcedureWithResultAsync();
            Console.WriteLine($"The result is: {result}" + Environment.NewLine +
                "Press any key to continue with the demonstration...");
        }

        /// <summary>
        /// Performs asynchronous operation and return arbitrary number
        /// </summary>
        /// <returns>Task with the int return type</returns>
        private static async Task<int> ProcedureWithResultAsync()
        {
            Console.WriteLine("ProcedureWithResultAsync has started...");
            await Task.Delay(2000);
            Console.WriteLine("ProcedureWithResultAsync has finished...");
            return 42;
        }
    }
}
